#!/usr/bin/perl
#############################################################################
#
# ppplag.pl -- PPP
# Vladi Belperchinov-Shabanski <cade@biscom.net> <cade@datamax.bg>
# http://www.biscom.net/~cade
# Any notes, comments, suggestions are welcome! Thank you!
#
# ChangeLog/History:
# 
# 04.Nov.2000 -- initial release
#
#############################################################################
# config section -- edit as you want/need
#############################################################################

$LOG = $ARGV[0] || "/var/log/messages";

#############################################################################
# config section end -- don't edit below...
# everything below this line is just code -- nothing much to see there :)
#############################################################################

if ( ! -r $LOG )
  {
  print "ppplag.pl: cannot read $LOG, cannot continue\n";
  exit;
  }

my $sep = "-" x 75 . "\n";

my $total_time = 0;
my $total_in   = 0;
my $total_out  = 0;

my $best_time  = 0;
my $best_in    = 0;
my $best_out   = 0;
my $best_speed_in  = 0;
my $best_speed_out = 0;

printf "%15s %8s %14s %14s %9s %9s\n", "",   "Time/hrs", "Bytes IN", "Bytes OUT", "Bps/IN", "Bps/OUT";

print $sep;

open( i, $LOG );
while(<i>)
  {
  if ( /Serial connection established/ )
    {
    my $this_start;
    my $this_time = 0;
    my $this_in   = 0;
    my $this_out  = 0;
 
    $this_start = $1 if /^(... .. ..:..:..)/;
 
    while(<i>)
      {
      last if /Exit/;
      $this_time = $1 if /Connect time ([\d\.]+) minutes/;
      if ( /Sent (\d+) bytes, received (\d+) bytes/ )
        {
        $this_out = $1;
        $this_in  = $2;
        };
      }
    next if $this_time == 0;

    my $this_time2 = sprintf "%s:%02s", int($this_time / 60), int($this_time % 60);
    my $this_speed_in  = int(( $this_in  ) / ( $this_time * 60 ));
    my $this_speed_out = int(( $this_out ) / ( $this_time * 60 ));
 
    printf "%15s %8s %14s %14s %9s %9s\n", $this_start, $this_time2, comma($this_in), comma($this_out), $this_speed_in, $this_speed_out;
 
    $best_time  = $this_time  if $this_time > $best_time;
    $best_in    = $this_in    if $this_in   > $best_in;
    $best_out   = $this_out   if $this_out  > $best_out;
    $best_speed_in  = $this_speed_in  if $this_speed_in  > $best_speed_in ;
    $best_speed_out = $this_speed_out if $this_speed_out > $best_speed_out;
 
    $total_time += $this_time;
    $total_in   += $this_in;
    $total_out  += $this_out;
    }
  }
close(i);
print $sep;

my $total_time2 = sprintf "%s:%02s", int($total_time / 60), int($total_time % 60);
my $best_time2  = sprintf "%s:%02s", int($best_time / 60), int($best_time % 60);

my $total_speed_in  = $total_time != 0 ? int(( $total_in  ) / ( $total_time * 60 )) : "n/a";
my $total_speed_out = $total_time != 0 ? int(( $total_out ) / ( $total_time * 60 )) : "n/a";

$best_speed_in  = "n/a" if $best_speed_in  == 0;
$best_speed_out = "n/a" if $best_speed_out == 0;

printf "%15s %8s %14s %14s %9s %9s\n", "Totals:", $total_time2, comma($total_in), comma($total_out), $total_speed_in, $total_speed_out;
printf "%15s %8s %14s %14s %9s %9s\n", "Best:",   $best_time2, comma($best_in), comma($best_out), $best_speed_in, $best_speed_out;

print $sep;


#############################################################################
# utility functions
#############################################################################

sub comma
{
  my $res = reverse shift;
  $res =~ s/(\d\d\d)/$1,/go;
  $res =~ s/,$//;
  $res = reverse $res;
}

#############################################################################
# eof
#############################################################################

